@@ -1,16 +1,17 @@
-from typing import Union
-
 from statefun_tasks.context import TaskContext
 from statefun_tasks.messages_pb2 import TaskRequest, TaskResult, TaskException, Pipeline, TupleOfAny
 from statefun_tasks.pipeline_helper import _PipelineHelper
-from statefun_tasks.serialisation import DefaultSerialiser
+from statefun_tasks.serialisation import DefaultSerialiser, pack_any
 from statefun_tasks.types import Task, Group
 
+from typing import Union
+
 
 class _Pipeline(object):
-    def __init__(self, pipeline: list, serialiser=None):
+    def __init__(self, pipeline: list, serialiser=None, is_fruitful=True):
         self._pipeline = pipeline
         self._serialiser = serialiser if serialiser is not None else DefaultSerialiser()
+        self._is_fruitful = is_fruitful
         self._pipeline_helper = _PipelineHelper(self._pipeline, self._serialiser)
 
     def to_proto(self) -> Pipeline:
@@ -29,13 +30,17 @@ class _Pipeline(object):
 
         return _Pipeline(pipeline, serialiser)
 
-    def begin(self, context: TaskContext, invoking_task: TaskRequest):
+    def is_empty(self):
+        return not any(self._pipeline_helper.get_initial_tasks())
+
+    def begin(self, context: TaskContext, invoking_task: TaskRequest, task_state):
         caller_id = context.get_caller_id()
 
         # 1. record all the continuations into a pipeline and save into state with caller id and address
         context.pipeline_state.id = context.get_task_id()
         context.pipeline_state.address = context.get_address()
         context.pipeline_state.pipeline.CopyFrom(self.to_proto())
+        context.pipeline_state.is_fruitful = self._is_fruitful
 
         if caller_id is not None:
             context.pipeline_state.caller_id = caller_id
@@ -55,23 +60,27 @@ class _Pipeline(object):
             # set extra pipeline related parameters
             self._add_pipeline_meta(context, caller_id, request)
 
-            self._serialiser.serialise_request(request, task.request, retry_policy=task.retry_policy)
+            self._serialiser.serialise_request(request, task.request, state=task_state, retry_policy=task.retry_policy)
 
             context.send_message(task.get_destination(), task.task_id, request)
 
     def resume(self, context: TaskContext, task_result_or_exception: Union[TaskResult, TaskException]):
         caller_id = context.get_caller_id()
-        task_results = context.pipeline_state.task_results
+        #task_results = context.pipeline_state.task_results
 
         # mark pipeline step as complete & record task result
-        self._pipeline_helper.mark_task_complete(caller_id, task_result_or_exception, task_results)
+        self._pipeline_helper.mark_task_complete(caller_id, task_result_or_exception)
 
         # get the next step of the pipeline to run (if any)
         _, next_step, group = self._pipeline_helper.get_next_step_in_pipeline(caller_id)
 
-        # if the group is complete the create the aggregate results as a list
-        if group is not None and group.is_complete():
-            task_result_or_exception = self._pipeline_helper.aggregate_group_results(group, task_results)
+        # if this task is part group then we need to record the results so we can aggregate later
+        if group is not None:
+            self._pipeline_helper.add_to_task_results(caller_id, task_result_or_exception, context.pipeline_state.task_results)
+
+            # once the group is complete aggregate the results
+            if group.is_complete():
+                task_result_or_exception = self._pipeline_helper.aggregate_group_results(group, context.pipeline_state.task_results)
 
         # if we got an exception then the next step is the finally_task if there is one (or none otherwise)
         if isinstance(task_result_or_exception, TaskException):
@@ -152,10 +161,14 @@ class _Pipeline(object):
         # finally emit the result (to egress, destination address or caller address)
         self._emit_result(context, task_request, task_result_or_exception)
 
-    @staticmethod
-    def _emit_result(context, task_request, task_result_or_exception):
+    def _emit_result(self, context, task_request, task_result_or_exception):
         # the result of this task is the result of the pipeline
         if isinstance(task_result_or_exception, TaskResult):
+
+            # if we are not a fruitful pipeline then zero out the result
+            if not context.storage.pipeline_state.is_fruitful:
+                task_result_or_exception.result.CopyFrom(pack_any(TupleOfAny()))
+
             context.storage.task_result = task_result_or_exception
         else:
             context.storage.task_exception = task_result_or_exception
