@@ -3,7 +3,7 @@ from statefun_tasks.utils import _gen_id, _is_tuple, _task_type_for
 from statefun_tasks.pipeline import _Pipeline
 from statefun_tasks.messages_pb2 import TaskRequest, Pipeline
 
-from typing import Union, Iterable
+from typing import Iterable
 
 
 def in_parallel(entries: list):
@@ -161,15 +161,17 @@ class PipelineBuilder(object):
         self._pipeline.append(task_entry)
         return self
 
-    def to_pipeline(self, serialiser=None):
+    def to_pipeline(self, serialiser=None, is_fruitful=True):
         """
         Concretises the builder into a pipeline
 
         :param option serialiser: the serialiser to use such as DefaultSerialiser
+        :param option is_fruitful: whether this pipeline is fruitful (i.e. returns a result). Default is True
+
         :return: a Flink Tasks pipeline
         """
         self.validate()
-        return _Pipeline(self._pipeline, serialiser=serialiser)
+        return _Pipeline(self._pipeline, serialiser=serialiser, is_fruitful=is_fruitful)
 
     def validate(self) -> 'PipelineBuilder':
         """
@@ -234,6 +236,14 @@ class PipelineBuilder(object):
         
         return list(yield_tasks(self))
 
+    def is_empty(self):
+        """
+        Tests if the pipeline contains any tasks
+
+        :return: true if empty, false otherwise
+        """
+        return not any(self.get_tasks())
+
     @staticmethod
     def _task_type_and_parameters_for(fun):
         try:
