@@ -70,12 +70,13 @@ class FlinkTasks(object):
         """
         self._bindings[f'__builtins.{type_name}'] = _FlinkTask(fun, self._serialiser, **params)
 
-    def register(self, fun, module_name=None, **params):
+    def register(self, fun, wrapper=None, module_name=None, **params):
         """
         Registers a Python function as a Flink Task.
         Normally you would attribute the function with @tasks.bind() instead
 
         :param fun: the python function
+        :param optional wrapper: if wrapping a task function with e.g. functools.wraps then pass the wrapper here
         :param optional module_name: the module name to register the task under which by default is the Python module name containing the function
         :param params: any additional parameters to the Flink Task (such as a retry policy)
         """
@@ -84,7 +85,7 @@ class FlinkTasks(object):
             raise ValueError("function instance must be provided")
 
         fun.type_name = _task_type_for(fun, module_name)
-        self._bindings[fun.type_name] = _FlinkTask(fun, self._serialiser, **params)
+        self._bindings[fun.type_name] = _FlinkTask(wrapper or fun, self._serialiser, **params)
 
     def bind(
         self, 
@@ -155,19 +156,23 @@ class FlinkTasks(object):
 
                 if message.is_type(TASK_REQUEST_TYPE):
                     task_input = message.as_type(TASK_REQUEST_TYPE)
+                    task_context.task_name = task_input.type
                     task_context.apply_task_meta(task_input)
                     await self._invoke_task(task_context, task_input)
 
                 elif message.is_type(TASK_ACTION_REQUEST_TYPE):
                     task_input = message.as_type(TASK_ACTION_REQUEST_TYPE)
+                    task_context.task_name = f'Action [{TaskAction.Name(task_input.action)}]'
                     self._invoke_action(task_context, task_input)
                     
                 elif message.is_type(TASK_RESULT_TYPE):
                     task_input = message.as_type(TASK_RESULT_TYPE)
+                    task_context.task_name = task_input.type
                     self._resume_pipeline(task_context, task_input)
 
                 elif message.is_type(TASK_EXCEPTION_TYPE):
                     task_input = message.as_type(TASK_EXCEPTION_TYPE)
+                    task_context.task_name = task_input.type
                     self._resume_pipeline(task_context, task_input)
 
                 else:
@@ -221,24 +226,24 @@ class FlinkTasks(object):
         context.storage.task_request = task_request
 
         flink_task = self.get_task(task_request.type)
-        task_result, task_exception, pipeline = await flink_task.run(context, task_request)
+        task_result, task_exception, pipeline, task_state = await flink_task.run(context, task_request)
 
         # if task returns a pipeline then start it
-        if pipeline is not None:
-            pipeline.begin(context, task_request) 
+        if pipeline is not None and not pipeline.is_empty():
+            pipeline.begin(context, task_request, task_state) 
 
         # else if we have a task result return it
         elif task_result is not None:
-                context.storage.task_result = task_result
-                self._emit_result(context, task_request, task_result)
+            context.storage.task_result = task_result
+            self._emit_result(context, task_request, task_result)
 
         # else if we have an task exception, attempt retry or return the error
         elif task_exception is not None:
-                if self._attempt_retry(context, task_request, task_exception):
-                    return  # we have triggered a retry so ignore the result of this invocation
+            if self._attempt_retry(context, task_request, task_exception):
+                return  # we have triggered a retry so ignore the result of this invocation
 
-                context.storage.task_exception = task_exception
-                self._emit_result(context, task_request, task_exception)
+            context.storage.task_exception = task_exception
+            self._emit_result(context, task_request, task_exception)
 
         _log.info(f'Finished {task_request.type}, {context}')
 
@@ -308,4 +313,8 @@ class FlinkTasks(object):
 
     def _fail(self, context, task_input, ex):
         task_exception = _create_task_exception(task_input, ex)
+
+        del context.storage.task_result
+        context.storage.task_exception = task_exception
+
         self._emit_result(context, task_input, task_exception)
